"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = exports.processDeploymentGroup = exports.processDeploymentTarget = void 0;
const core_1 = require("@takomo/core");
const util_1 = require("@takomo/util");
const child_process_1 = require("child_process");
const cockatiel_1 = require("cockatiel");
const util_2 = require("util");
const execP = util_2.promisify(child_process_1.exec);
const getDeploymentRole = (currentIdentity, group, target, roleName) => {
    const deploymentRole = target.deploymentRole ?? group.deploymentRole;
    if (deploymentRole) {
        return deploymentRole;
    }
    const deploymentRoleName = roleName ?? target.deploymentRoleName ?? group.deploymentRoleName;
    if (!deploymentRoleName) {
        return undefined;
    }
    const deploymentAccountId = target.accountId ?? currentIdentity.accountId;
    return {
        iamRoleArn: `arn:aws:iam::${deploymentAccountId}:role/${deploymentRoleName}`,
    };
};
const processDeploymentTarget = async (props, group, target, timer, state) => {
    const { io, ctx, input } = props;
    io.info(`Run deployment target: ${target.name}`);
    if (state.failed) {
        timer.stop();
        return {
            message: "Cancelled",
            status: "CANCELLED",
            success: false,
            name: target.name,
            timer,
        };
    }
    const currentIdentity = await ctx.credentialManager.getCallerIdentity();
    const deploymentRole = getDeploymentRole(currentIdentity, group, target, input.roleName);
    const credentialManager = deploymentRole
        ? await ctx.credentialManager.createCredentialManagerForRole(deploymentRole.iamRoleArn)
        : ctx.credentialManager;
    if (target.accountId) {
        const identity = await credentialManager.getCallerIdentity();
        if (identity.accountId !== target.accountId) {
            throw new util_1.TakomoError(`Current credentials belong to AWS account ${identity.accountId}, but the deployment target can be run only against account: ${target.accountId}`);
        }
    }
    const env = {
        ...util_1.deepCopy(process.env),
        TKM_TARGET: target.name,
    };
    const credentials = await credentialManager.getCredentials();
    env.AWS_ACCESS_KEY_ID = credentials.accessKeyId;
    env.AWS_SECRET_ACCESS_KEY = credentials.secretAccessKey;
    env.AWS_SESSION_TOKEN = credentials.sessionToken;
    env.AWS_SECURITY_TOKEN = credentials.sessionToken;
    const cwd = ctx.projectDir;
    try {
        const { stdout } = await execP(input.command, { cwd, env });
        timer.stop();
        io.infoText("Command output:", stdout);
        return {
            status: "SUCCESS",
            name: target.name,
            success: true,
            message: "Success",
            timer,
        };
    }
    catch (e) {
        state.failed = true;
        io.infoText("Command output:", e.stdout);
        timer.stop();
        return {
            status: "FAILED",
            name: target.name,
            success: false,
            message: "Error",
            error: e,
            timer,
        };
    }
};
exports.processDeploymentTarget = processDeploymentTarget;
const convertToOperation = (props, group, timer, state, target, results, policy, listener) => () => policy.execute(async () => {
    await listener.onTargetBegin();
    const result = await exports.processDeploymentTarget(props, group, target, timer.startChild(target.name), state);
    results.push(result);
    await listener.onTargetComplete();
    return result;
});
/**
 * @hidden
 */
const processDeploymentGroup = async (props, group, timer, state, listener) => {
    const { io, input } = props;
    const { concurrentTargets } = input;
    io.info(`Execute deployment group: ${group.path}`);
    const results = new Array();
    const deploymentPolicy = cockatiel_1.Policy.bulkhead(concurrentTargets, 10000);
    const operations = group.targets.map((target) => convertToOperation(props, group, timer, state, target, results, deploymentPolicy, listener));
    await Promise.all(operations.map((o) => o()));
    timer.stop();
    return {
        ...core_1.resolveCommandOutputBase(results),
        results,
        path: group.path,
        timer,
    };
};
exports.processDeploymentGroup = processDeploymentGroup;
const run = async (props) => {
    const { input, plan, listener } = props;
    const childTimer = input.timer.startChild("run");
    const results = new Array();
    const state = { failed: false };
    for (const group of plan.groups) {
        const result = await exports.processDeploymentGroup(props, group, childTimer.startChild(group.name), state, listener);
        results.push(result);
    }
    childTimer.stop();
    return {
        ...core_1.resolveCommandOutputBase(results),
        results,
        timer: childTimer,
    };
};
exports.run = run;
//# sourceMappingURL=run.js.map